#
# fishing.R, 21 Aug 22
#
# Data from:
# https://jira.lsstcorp.org/projects/DM/issues
# Downloaded on 17 Aug 22 at 00:30

library("colorspace")
library("gnm")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")

pal_col=rainbow(2)



# Info on Sprint column
get_sprint_info=function(df)
{
si=data.frame(total_SP=sum(df$Story.Points, na.rm=TRUE),
		num_Stories=nrow(df),
		story_with_SP=nrow(subset(df, (!is.na(Story.Points)) & (Story.Points > 0))),
		resolved_first=min(df$Resolved, na.rm=TRUE),
		resolved_last=max(df$Resolved, na.rm=TRUE)
		)
return(si)
}


# Totals for all Sprints
get_all_sprints_SP_story=function(df)
{
SP_finished=subset(df, last_sprint)
si=data.frame(total_SP=sum(df$SP, na.rm=TRUE),
		total_SP_finished=sum(SP_finished$SP, na.rm=TRUE),
		num_Stories=nrow(df),
		num_Stories_finished=nrow(SP_finished),
		story_with_SP=nrow(subset(df, (!is.na(SP)) & (SP > 0))),
		resolved_first=min(SP_finished$Resolved, na.rm=TRUE),
		resolved_last=max(SP_finished$Resolved, na.rm=TRUE)
		)
return(si)
}


# Info on all Sprints for each story
get_all_sprints_stories=function(s_df)
{
# Non-empty string is assumed to be an assigned sprint
num_sprints=ddply(s_df, .(Issue.id), function(df)
					{
					str=paste(df[ , spr_cols], collapse=" | ")
					# Remove noise created by empty columns
					str=gsub(" \\| NA", "", str)
					str=gsub(" \\|(  \\|)+", " |", str)
					data.frame(count=length(which(df[ , spr_cols] != "")),
							str=gsub(" \\| $", "", str),
							SP=df$Story.Points)
					})
return(num_sprints)
}


# Create one row for every sprint that a story is assigned to
mk_sprint_stories=function(df)
{
sprint_str=as.vector(t(df[ , spr_cols]))
sprint_str=sprint_str[which((sprint_str != "") & !is.na(sprint_str))]

story_spr=data.frame(sprint_str,
			sprint_num=1:length(sprint_str),
			SP=df$Story.Points,
			Resolved=df$Resolved)
story_spr$last_sprint=(story_spr$sprint_num == length(sprint_str))
return(story_spr)
}

dm=read.csv("jira-SP/DM500.csv.xz", as.is=TRUE)
table(dm$Story.Points)

plot(count(dm$Story.Points), log="xy", xlim=c(1, 1000))

# How many columns are not all NAs?
length(which(colSums(is.na(dm) == FALSE) == 0))

col_names=names(dm)

done=subset(dm, Status == "Done")
done$Created=as.Date(done$Created, format="%d/%b/%y")
done$Updated=as.Date(done$Updated, format="%d/%b/%y")
done$Resolved=as.Date(done$Resolved, format="%d/%b/%y")
# done$Start.date=as.Date(done$Start.date, format="%d/%b/%y") # Lots are NA
# done$End.date=as.Date(done$End.date, format="%d/%b/%y") # Lots are NA

sprints=subset(done, (!is.na(Sprint)) & (Sprint != ""))
spr_cols=which(grepl("Sprint", col_names))


# Number of stories assigned a given number of story-points
pal_col=rainbow(4)

sp_cnt=count(sprints$Story.Points)
sp_rnd_cnt=count(round(sprints$Story.Points))
plot(sp_cnt, log="y",
	xlim=c(1, 50),
	xlab="Story points", ylab="Stories")
points(sp_rnd_cnt, col=pal_col[1])

# Single exponential model
# rsp_mod=glm(log(freq) ~ x, data=sp_rnd_cnt, subset=(x>0) & (x < 50))
# summary(rsp_mod)
# pred=predict(rsp_mod, newdata=data.frame(x=xbounds))
# lines(xbounds, exp(pred), col=pal_col[2])

# Biexponential model
rsp_mod=gnm(freq ~ instances(Mult(1, Exp(x)), 2)-1,
                data=sp_rnd_cnt, verbose=TRUE, trace=FALSE,
		subset=(x>0) & (x < 50),
                start=c(3000, -0.10, 500, -0.001))
summary(rsp_mod)
pred=predict(rsp_mod, newdata=data.frame(x=xbounds))
lines(xbounds, pred, col=pal_col[2], lwd=2.1)

exp_coef=as.numeric(coef(rsp_mod))

lines(xbounds, exp_coef[1]*exp(exp_coef[2]*xbounds), col=pal_col[3])
lines(xbounds, exp_coef[3]*exp(exp_coef[4]*xbounds), col=pal_col[4])

legend("topright", legend=c("Data points", "Fitted biexponential",
				"1st exponential", "2nd exponential", "Non-integer SP"),
                        fill=c(pal_col, "black"), border="white", bty="n", cex=1.3)

nr=subset(sp_cnt, !(x %in% 1:200))    
# sum(nr$freq)
# [1] 2405
rr=subset(sp_cnt, (x %in% 1:200))  
# sum(rr$freq)                     
# [1] 2405
sum(nr$freq)/sum(rr$freq)        
# [1] 0.2771695

a_cnt=count(sprints$Assignee)
r_cnt=count(sprints$Reporter)
c_cnt=count(sprints$Creator)

plot(sort(a_cnt$freq), log="y", col=pal_col[1])
points(sort(r_cnt$freq), col=pal_col[2])
points(sort(c_cnt$freq), col=pal_col[3])


#
# Look at sprints in just the Sprint column
spr_story=ddply(sprints, .(Sprint), get_sprint_info)

SP_story=count(spr_story$num_Stories)
plot(SP_story, log="xy", col=pal_col[1],
	xaxs="i",
	xlim=c(0.9, 70),
	xlab="Stories", ylab="Sprints")

SP_mod=glm(log(freq) ~ log(x), data=SP_story, subset=(x < 40))
summary(SP_mod)

plot(spr_info$story_with_SP, spr_info$total_SP, log="xy", col=pal_col[1],
	xlab="Stories (with SP)", ylab="Total SP in story")
story_mod=glm(log(total_SP) ~ log(story_with_SP), data=spr_info,
			subset=(story_with_SP > 0))
summary(story_mod)
xbounds=1:100
pred=predict(story_mod, newdata=data.frame(story_with_SP=xbounds))
lines(xbounds, exp(pred), col=pal_col[2])

#
# Sprint info associated with each story
spr_with_SP=subset(sprints, (!is.na(Story.Points)) & (Story.Points > 0))

sprints_all=get_all_sprints_stories(sprints)

sprints_wSP=get_all_sprints_stories(subset(sprints, (!is.na(Story.Points)) & (Story.Points > 0)))
spr_wSP_count=count(sprints_wSP$count)

plot(jitter(sprints_all$SP), jitter(sprints_all$count), log="x",
		xlim=c(1, 30),
		xlab="Story points", ylab="Sprints")
lines(loess.smooth(sprints_all$SP, sprints_all$count, span=0.5))

#
# How many sprints did each story get assigned to?
pal_col=rainbow(2)
spr_count=count(sprints_all$count)
plot(spr_count, log="xy", col=pal_col[1],
	xlab="Sprints", ylab="Stories")
# points(spr_wSP_count, col="green")

spr_mod=glm(log(freq) ~ log(x), data=spr_count)
summary(spr_mod)

pred=predict(spr_mod)
lines(spr_count$x, exp(pred), col=pal_col[2])


sprint_stories=ddply(sprints, .(Issue.id), mk_sprint_stories)
sprint_totals=ddply(sprint_stories, .(sprint_str), get_all_sprints_SP_story)

sprint_totals$total_SP_ratio=sprint_totals$total_SP_finished/sprint_totals$total_SP
sprint_totals$num_Stories_ratio=sprint_totals$num_Stories_finished/sprint_totals$num_Stories

numS_cnt=count(sprint_totals$num_Stories)
plot(numS_cnt, col=pal_col[2],
	xlab="Stories", ylab="Sprints")
lines(loess.smooth(numS_cnt$x, numS_cnt$freq, span=0.3), col=pal_col[2])
numSf_cnt=count(sprint_totals$num_Stories_finished)
points(numSf_cnt, col=pal_col[1])
lines(loess.smooth(numSf_cnt$x, numSf_cnt$freq, span=0.3), col=pal_col[1])
 
legend("topright", legend=c("Implemented", "Assigned"),
                        fill=pal_col, border="white", bty="n", cex=1.3)


plot(sprint_totals$total_SP, sprint_totals$total_SP_ratio, col=pal_col[1],
	xlab="Total story-points assigned to sprint", ylab="Story-points implemented/assigned")
lines(loess.smooth(sprint_totals$total_SP, sprint_totals$total_SP_ratio, span=0.5), col=pal_col[2])

plot(sprint_totals$num_Stories, sprint_totals$num_Stories_ratio, col=pal_col[1],
	xlab="Stories assigned to sprint", ylab="Stories implemented/assigned")
lines(loess.smooth(sprint_totals$num_Stories, sprint_totals$num_Stories_ratio, span=0.5), col=pal_col[2])


# The following was used to merge the files of 10K issues downloaded from the repo
#
# # Column order/contents varies between files
# # So file contents needs to be merge'd
# DM=list.files("jira-SP", "DM-.*.csv", full.names=TRUE)
# 
# dm=NULL
# dummy=sapply(DM, function(X)
# 			{
# 			f=read.csv(X, as.is=TRUE)
# 			dm<<-merge(dm, f, all=TRUE, sort=FALSE)
# 			return(0)
# 			}
# 		)
# dm500=dm[ , 1:400]
#
# Remove "all NA" columns
# dm500[ , which(colSums(is.na(dm500) == FALSE) == 0)]=NULL

